class Trim
	attr_accessor :trim_name, :exterior, :interior, :performance, :fuel_economy
                         
	def initialize()
	  @exterior = Struct::ExteriorStats.new
	  @interior = Struct::InteriorStats.new
    @performance = Struct::PerformanceStats.new
    @fuel_economy = Struct::FuelEconomyStats.new
  end


  def mpg_combined
    (self.mpg_city + self.mpg_highway) / 2
  end
  
  def mpg_city
    divider_index = @fuel_economy.epa_mileage_estimates.index('C')
    Integer(@fuel_economy.epa_mileage_estimates[0..divider_index-1])
  end
  
  def mpg_highway
    divider_index = @fuel_economy.epa_mileage_estimates.index("/") + 1
    end_index = @fuel_economy.epa_mileage_estimates.index('H')
    Integer(@fuel_economy.epa_mileage_estimates[divider_index..end_index-1].strip)
  end
  
  def highway_range
    self.mpg_highway * @fuel_economy.fuel_tank_capacity
  end
  
  def city_range
    self.mpg_city * @fuel_economy.fuel_tank_capacity
  end
  
  def get_stat(type)
    if @performance.members.include? type.to_sym; return @performance[type]; end
    if @interior.members.include? type.to_sym; return @interior[type]; end
    if @exterior.members.include? type.to_sym; return @exterior[type]; end
    if @fuel_economy.members.include? type.to_sym; return @fuel_economy[type]; end
  end
  
  def available_stats
    @performance.members.each do |stat_name|
      unless @performance[stat_name].nil?;puts "performance.#{stat_name.to_s}"end
    end
    @interior.members.each do |stat_name|
      unless @interior[stat_name].nil?;puts "interior.#{stat_name.to_s}"end
    end
    @exterior.members.each do |stat_name|
      unless @exterior[stat_name].nil?;puts "exterior.#{stat_name.to_s}"end
    end
    @fuel_economy.members.each do |stat_name|
      unless @fuel_economy[stat_name].nil?;puts "fuel_economy.#{stat_name.to_s}"end
    end
  end
  
  
  def engine
    puts "#{@performance.base_engine_size} #{@performance.base_engine_type} - with #{@performance.horsepower} at #{@performance.horsepower_rpm}rpm" + 
    " and #{@performance.torque} of torque at #{@performance.torque_rpm}rpm"
  end
  
	def to_s
	  @trim_name
	end
	
end
